from aleksis.core.util.apps import AppConfig


class DefaultConfig(AppConfig):
    name = "aleksis.apps.order"
    verbose_name = "AlekSIS — Order (Manage orders)"

    urls = {
        "Repository": "https://edugit.org/hansegucker/AlekSIS-App-Order",
    }
    licence = "EUPL-1.2+"
    copyright_info = (
        ([2020, 2021], "Jonathan Weth", "dev@jonathanweth.de"),
        ([2021], "Hangzhi Yu", "yuha@katharineum.de"),
    )



from django.contrib import admin

from guardian.admin import GuardedModelAdminMixin

from .models import Item, Order, OrderForm, OrderItem, ProcessingOption, ProcessingOptionPrice


class ItemInline(admin.TabularInline):
    model = Item


class ItemAdmin(admin.ModelAdmin):
    model = Item


admin.site.register(Item, ItemAdmin)


class OrderFormAdmin(GuardedModelAdminMixin, admin.ModelAdmin):
    model = OrderForm


admin.site.register(OrderForm, OrderFormAdmin)


class OrderItemInline(admin.TabularInline):
    model = OrderItem


class OrderAdmin(admin.ModelAdmin):
    model = Order
    readonly_fields = ["confirm_key", "form"]
    list_display = ["__str__", "submitted", "confirmed", "paid", "sent"]
    inlines = [OrderItemInline]


admin.site.register(Order, OrderAdmin)


class ProcessingOptionPriceInline(admin.TabularInline):
    model = ProcessingOptionPrice


class ProcessingOptionAdmin(admin.ModelAdmin):
    model = ProcessingOption
    inlines = [ProcessingOptionPriceInline]


admin.site.register(ProcessingOption, ProcessingOptionAdmin)



# Generated by Django 3.1.3 on 2020-12-18 13:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0004_order_notes'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='order',
            options={'permissions': [('manage_orders', 'Can manage orders')], 'verbose_name': 'Reihenfolge', 'verbose_name_plural': 'Orders'},
        ),
    ]



from django.utils.translation import ugettext_lazy as _

MENUS = {
    "NAV_MENU_CORE": [
        {
            "name": _("Orders"),
            "url": "#",
            "svg_icon": "mdi:cart-outline",
            "root": True,
            "validators": [
                (
                    "aleksis.core.util.predicates.permission_validator",
                    "order.show_menu",
                ),
            ],
            "submenu": [
                {
                    "name": _("List"),
                    "url": "list_orders",
                    "svg_icon": "mdi:receipt-outline",
                    "validators": [
                        (
                            "aleksis.core.util.predicates.permission_validator",
                            "order.view_orders",
                        ),
                    ],
                },
                {
                    "name": _("Pick up"),
                    "url": "pick_up_order",
                    "svg_icon": "mdi:receipt-outline",
                    "validators": [
                        (
                            "aleksis.core.util.predicates.permission_validator",
                            "order.view_orders",
                        ),
                    ],
                },
            ],
        }
    ]
}



# Generated by Django 3.1.3 on 2020-12-18 13:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0005_auto_20201218_1449'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderform',
            name='access_code',
            field=models.CharField(blank=True, max_length=2555, verbose_name='Access code'),
        ),
    ]



from tempfile import TemporaryDirectory
from typing import Any, Dict

from django.contrib import messages
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse
from django.http.response import FileResponse, HttpResponseNotFound
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse_lazy
from django.utils.translation import gettext as _
from django.views.generic import FormView
from django.views.generic.base import View
from django.views.generic.detail import DetailView

from django_filters.views import FilterView
from django_tables2 import SingleTableMixin
from rules.contrib.views import PermissionRequiredMixin

from aleksis.apps.order.filters import OrderFilter
from aleksis.apps.order.pdf import generate_address_labels, generate_order_labels
from aleksis.apps.order.tables import OrderTable
from aleksis.core.mixins import AdvancedDeleteView, AdvancedEditView
from aleksis.core.util.core_helpers import has_person, queryset_rules_filter
from aleksis.core.util.pdf import render_pdf

from .forms import (
    AccessForm,
    NotesForm,
    OrderActionForm,
    OrderFormForm,
    OrderItemFormSet,
    PickUpForm,
    ProcessingOptionForm,
    ShippingAddressForm,
)
from .models import Order, OrderForm, OrderItem


class OrderFormView(DetailView):
    model = OrderForm
    template_name = "order/form.html"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        form_access_granted_key = f"order_access_{self.object.pk}"

        if self.object.closed:
            return redirect("index")

        access_granted = (
            has_person(self.request)
            or request.session.get("order")
            or request.session.get(form_access_granted_key)
            or not self.object.access_code.strip()
        )
        if access_granted:
            order = self.request.session.get("order")
            try:
                order = Order.objects.get(pk=order)
            except Order.DoesNotExist:
                order = None

            form = OrderFormForm(self.request.POST or None, instance=order)
            formset = OrderItemFormSet(
                self.request.POST or None,
                queryset=self.object.available_items.all(),
                prefix="order_item",
            )

            # Load order from session
            if order:
                counts_by_pk = {item.item.pk: item.count for item in order.items.all()}
                for item_form in formset:
                    item_form.fields["count"].initial = counts_by_pk.get(item_form.instance.pk, 0)

            context["form"] = form
            context["formset"] = formset

        if not access_granted:
            # Show access code form
            access_form = AccessForm(request.POST or None)
            context["access_form"] = access_form
            if request.method == "POST" and access_form.is_valid():
                if (
                    self.object.access_code.lower().strip()
                    == access_form.cleaned_data["access_code"]
                ):
                    request.session[form_access_granted_key] = True
                    return redirect("order_form", self.object.pk)
                else:
                    messages.error(request, _("Please enter the correct access code."))

            return render(request, "order/access.html", context)

        person = None
        if has_person(self.request):
            person = self.request.user.person
            form.fields["full_name"].disabled = True
            form.fields["full_name"].initial = person.addressing_name
            if person.email:
                form.fields["email"].disabled = True
                form.fields["email"].initial = person.email

        if self.request.method == "POST":
            items = []
            if form.is_valid():
                full_name = person.addressing_name if person else form.cleaned_data["full_name"]
                email = (
                    person.email if getattr(person, "email", None) else form.cleaned_data["email"]
                )

                # Get items
                if formset.is_valid():
                    total_count_of_items = 0
                    for item_form in formset:
                        # Calculate count
                        count = item_form.cleaned_data["count"]
                        total_count_of_items += count

                        if count > 0:
                            order_item = None

                            # Check for existing order item
                            if order:
                                try:
                                    order_item = OrderItem.objects.get(
                                        item=item_form.instance, order=order
                                    )
                                except OrderItem.DoesNotExist:
                                    pass

                            # New order item
                            if not order_item:
                                order_item = OrderItem(item=item_form.instance, count=count)

                            order_item.new_count = count
                            items.append(order_item)

                    # Check if there is at least one item
                    if total_count_of_items <= 0:
                        messages.error(self.request, _("You must order at least one item."))
                        return self.render_to_response(context)

                    # Get or create order
                    if not order:
                        order = Order.objects.create(
                            form=self.object, full_name=full_name, email=email
                        )
                    else:
                        order.full_name = full_name
                        order.email = email
                        order.save()

                    # Save order items
                    for item in items:
                        item.order = order
                        item.count = item.new_count
                        item.save()

                    # Set session and redirect
                    self.request.session["order"] = order.pk
                    return redirect("order_form_2", self.object.pk)
        # TODO DON'T GET FULL NAME AND EMAIL FROM FORM IF USER IS LOGGED IN

        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderForm2View(DetailView):
    model = OrderForm
    template_name = "order/form2.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.object.pk)
        if self.order.form != self.get_object():
            return redirect("order_form_2", self.order.form.pk)

        if hasattr(self, "order"):
            context["order"] = self.order

            processing_form = ProcessingOptionForm(
                self.request.POST or None,
                initial={"processing_option": self.order.processing_option},
            )
            context["processing_form"] = processing_form

            processing_options = self.order.form.available_processing_options.all()
            for processing_option in processing_options:
                processing_option.price = processing_option.get_price(self.order.items_count)

            context["processing_options"] = processing_options

            shipping_address_form = ShippingAddressForm(
                self.request.POST or None,
                initial={
                    "full_name": self.order.shipping_full_name,
                    "second_address_row": self.order.second_address_row,
                    "street": self.order.street,
                    "housenumber": self.order.housenumber,
                    "plz": self.order.plz,
                    "place": self.order.place,
                },
            )
            context["shipping_address_form"] = shipping_address_form

            if self.request.method == "POST":
                if processing_form.is_valid():
                    processing_option = processing_form.cleaned_data["processing_option"]

                    if not processing_option.address_necessary or shipping_address_form.is_valid():
                        # WITH SHIPPING ADDRESS
                        self.order.processing_option = processing_option

                        if processing_option.address_necessary:
                            self.order.shipping_full_name = shipping_address_form.cleaned_data[
                                "full_name"
                            ]
                            self.order.second_address_row = shipping_address_form.cleaned_data[
                                "second_address_row"
                            ]
                            self.order.street = shipping_address_form.cleaned_data["street"]
                            self.order.housenumber = shipping_address_form.cleaned_data[
                                "housenumber"
                            ]
                            self.order.plz = shipping_address_form.cleaned_data["plz"]
                            self.order.place = shipping_address_form.cleaned_data["place"]

                        self.order.save()

                        return redirect("order_form_3", self.order.form.pk)
                else:
                    messages.error(request, _("Please select a processing option."))
        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderForm3View(DetailView):
    model = OrderForm
    template_name = "order/form3.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.object.pk)
        if self.order.form != self.get_object():
            return redirect("order_form_3", self.order.form.pk)

        context["order"] = self.order
        notes_form = NotesForm(request.POST or None, initial={"notes": self.order.notes})
        context["notes_form"] = notes_form
        if notes_form.is_valid():
            self.order.notes = notes_form.cleaned_data["notes"]
            self.order.save()
            return redirect("order_finished", self.order.form.pk)

        return self.render_to_response(context)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)


class OrderFinishedView(DetailView):
    model = OrderForm
    template_name = "order/finished.html"

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        order = self.request.session.get("order")
        try:
            self.order = Order.objects.get(pk=order)
        except Order.DoesNotExist:
            return redirect("order_form", self.get_object().pk)
        r = super().get(request, *args, **kwargs)
        if self.order.form != self.get_object():
            return redirect("order_finished", self.order.form.pk)
        return r

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        if hasattr(self, "order"):
            context["order"] = self.order
            self.order.submitted = True
            self.order.save()
            self.order.send_overview(self.request)
            form_access_granted_key = f"order_access_{self.order.form.pk}"

            del self.request.session["order"]
            if form_access_granted_key in self.request.session:
                del self.request.session[form_access_granted_key]
        return context


class OrderConfirmView(View):
    def get(self, request: HttpRequest, key, **kwargs: Any) -> HttpResponse:
        try:
            self.order = Order.objects.get(confirm_key=key)
        except Order.DoesNotExist:
            return HttpResponseNotFound()
        if not self.order.confirmed:
            self.order.confirm(request)
        return render(request, "order/confirmed.html", context={"order": self.order})


class OrderDetailView(PermissionRequiredMixin, DetailView):
    model = Order
    permission_required = "order.manage_order"
    template_name = "order/detail.html"


class OrderEditView(PermissionRequiredMixin, AdvancedEditView):
    model = Order
    fields = (
        "full_name",
        "email",
        "notes",
        "submitted",
        "confirmed",
        "paid",
        "sent",
        "processing_option",
        "shipping_full_name",
        "second_address_row",
        "street",
        "housenumber",
        "plz",
        "place",
    )
    permission_required = "order.manage_order"
    template_name = "order/edit.html"
    success_message = _("The order has been changed successfully.")
    success_url = reverse_lazy("list_orders")

    def post(self, request, *args, **kwargs):
        r = super().post(request, *args, **kwargs)

        order = self.get_object()

        if request.POST.get("mark-as-paid"):
            order.paid = True
            order.save()

        elif request.POST.get("mark-as-paid-email"):
            order.send_pay_confirmation()

        return r


class OrderDeleteView(PermissionRequiredMixin, AdvancedDeleteView):
    model = Order
    permission_required = "order.manage_order"
    template_name = "core/pages/delete.html"
    success_message = _("The order has been deleted successfully.")
    success_url = reverse_lazy("list_orders")


class OrderListView(PermissionRequiredMixin, SingleTableMixin, FilterView):
    table_class = OrderTable
    model = Order
    template_name = "order/list.html"
    permission_required = "order.view_orders"
    table_pagination = {"per_page": 50}

    filterset_class = OrderFilter

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["table"].request = self.request
        context["forms"] = queryset_rules_filter(
            self.request.user, OrderForm.objects.all(), "order.manage_orders_of_form"
        )
        self.action_form = OrderActionForm(
            self.request, self.request.POST or None, queryset=self.get_queryset()
        )
        context["action_form"] = self.action_form
        return context

    def post(self, request, *args, **kwargs):
        r = self.get(request, *args, **kwargs)

        if "action" in self.request.POST and self.action_form.is_valid():
            self.action_form.execute()

        elif request.POST.get("all-action"):
            if request.POST.get("print"):
                return render_pdf(request, "order/list_print.html", self.get_context_data())
            elif request.POST.get("packing-lists"):
                return render_pdf(request, "order/packing_lists.html", self.get_context_data())

        # Action for a single object
        elif request.POST.get("single-action-pk"):
            order = get_object_or_404(Order, pk=request.POST.get("single-action-pk"))

            if request.POST.get("mark-as-paid"):
                order.paid = True
                order.save()

            elif request.POST.get("mark-as-paid-email"):
                order.send_pay_confirmation()

            elif request.POST.get("mark-as-sent"):
                order.sent = True
                order.save()

        # Actions for all objects
        elif request.POST.get("all-action"):
            if request.POST.get("mark-as-paid"):
                self.object_list.update(paid=True)

            elif request.POST.get("mark-as-paid-email"):
                self.object_list.update(paid=True)
                for order in self.object_list:
                    order.send_pay_confirmation()
            elif request.POST.get("mark-as-sent"):
                self.object_list.update(sent=True)

            elif request.POST.get("labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_order_labels(self.object_list, temp_dir)
                    f = open(filename, "rb")
                    return FileResponse(f, filename="labels.pdf")
            elif request.POST.get("address-labels"):
                with TemporaryDirectory() as temp_dir:
                    filename = generate_address_labels(self.object_list, temp_dir)
                    f = open(filename, "rb")
                    return FileResponse(f, filename="labels.pdf")
        return r


class OrderPickUpFormView(PermissionRequiredMixin, FormView):
    template_name = "order/pickup.html"
    permission_required = "order.view_orders"
    form_class = PickUpForm

    def post(self, request, *args, **kwargs):
        if request.POST.get("order_id_action"):
            try:
                order_id = int(request.POST["order_id_action"])
            except ValueError:
                raise SuspiciousOperation()
            try:
                order = Order.objects.get(pk=order_id)
            except Order.DoesNotExist:
                raise SuspiciousOperation()

            if request.POST.get("abort"):
                return self.get(request, *args, **kwargs)
            elif request.POST.get("collected") and order.paid and not order.sent:
                order.sent = True
                order.save()
                messages.success(request, _("The order was successfully marked as collected."))
            elif request.POST.get("manually-paid-collected") and not order.paid and not order.sent:
                order.sent = True
                order.paid = True
                order.notes += _("\nManually paid")
                order.save()
                messages.success(
                    request, _("The order was successfully marked as manually paid and collected.")
                )

            elif request.POST.get("paid-collected") and not order.paid and not order.sent:
                order.sent = True
                order.paid = True
                order.save()
                messages.success(
                    request, _("The order was successfully marked as normally paid and collected.")
                )

        return super().post(request, *args, **kwargs)

    def form_valid(self, form) -> HttpResponse:
        context = self.get_context_data()
        order = form.cleaned_data["order_id"]
        print(order)
        context["order"] = order
        context["form"] = PickUpForm()

        return self.render_to_response(context)



from uuid import uuid4

from django.core.validators import MinValueValidator
from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from ckeditor_uploader.fields import RichTextUploadingField
from model_utils.models import TimeStampedModel
from templated_email import send_templated_mail

from aleksis.core.mixins import ExtensibleModel


class Item(ExtensibleModel):
    short_name = models.CharField(max_length=255, verbose_name=_("Short name"), unique=True)
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    notice = RichTextUploadingField(
        verbose_name=_("Notice"), help_text=_("Shown below the item"), blank=True
    )
    price = models.FloatField(validators=[MinValueValidator(0.0)])
    max_count = models.IntegerField(
        default=0, validators=[MinValueValidator(0)], verbose_name=_("Maximum count of items")
    )

    def __str__(self):
        return self.short_name

    class Meta:
        verbose_name = _("Item")
        verbose_name_plural = _("Items")


class ProcessingOption(ExtensibleModel):
    short_name = models.CharField(max_length=255, verbose_name=_("Short name"), unique=True)
    name = models.CharField(max_length=255, verbose_name=_("Name"))
    notice = RichTextUploadingField(
        verbose_name=_("Notice"), help_text=_("Shown below the option"), blank=True
    )
    email_notice = RichTextUploadingField(
        verbose_name=_("Email notice"), help_text=_("Shown in the confirmation email"), blank=True
    )
    address_necessary = models.BooleanField(default=False, verbose_name=_("Adress necessary"))

    def get_price(self, count):
        for price in self.prices.order_by("price"):
            if price.min_count <= count <= price.max_count:
                return price.price
        return 0

    def __str__(self):
        return self.short_name


class ProcessingOptionPrice(ExtensibleModel):
    option = models.ForeignKey(
        ProcessingOption, models.CASCADE, related_name="prices", verbose_name=_("Processing option")
    )
    min_count = models.IntegerField(
        verbose_name=_("Min count of items"), validators=[MinValueValidator(0)]
    )
    max_count = models.IntegerField(
        verbose_name=_("Max count of items"), validators=[MinValueValidator(0)]
    )
    price = models.FloatField(validators=[MinValueValidator(0.0)])

    def __str__(self):
        return f"{self.option}: {self.min_count} <= x <= {self.max_count}"


class OrderForm(ExtensibleModel):
    title = models.CharField(max_length=255, verbose_name=_("Form title"))
    available_items = models.ManyToManyField(
        to=Item, related_name="forms", verbose_name=_("Available items")
    )
    available_processing_options = models.ManyToManyField(
        to=ProcessingOption, related_name="forms", verbose_name=_("Available processing options")
    )
    help_text = RichTextUploadingField(
        verbose_name=_("Help text"), help_text=_("Shown in the form's footer"), blank=True
    )
    from_email = models.EmailField(verbose_name=_("From email address"))
    from_name = models.CharField(verbose_name=_("From name"), max_length=255)
    access_code = models.CharField(max_length=2555, blank=True, verbose_name=_("Access code"))
    closed = models.BooleanField(default=False, verbose_name=_("Form closed for orders"))
    sender = models.TextField(blank=True, verbose_name=_("Sender"))

    def __str__(self):
        return self.title

    @property
    def email_sender(self):
        return f"{self.from_name} <{self.from_email}>"

    @property
    def total(self):
        return sum([order.total for order in self.orders.all()])

    @property
    def items_count(self):
        return sum([order.items_count for order in self.orders.all()])

    @property
    def confirmed_count(self):
        return sum([order.items_count for order in self.orders.all() if order.confirmed])

    @property
    def paid_count(self):
        return sum([order.items_count for order in self.orders.all() if order.paid])

    @property
    def sent_count(self):
        return sum([order.items_count for order in self.orders.all() if order.sent])

    @property
    def annotated_items(self):
        item_list = []
        for item in self.available_items.all():
            item.items_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(order__form=self, item=item)
                ]
            )
            item.confirmed_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__confirmed=True
                    )
                ]
            )
            item.paid_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__paid=True
                    )
                ]
            )
            item.sent_count = sum(
                [
                    order_item.count
                    for order_item in OrderItem.objects.filter(
                        order__form=self, item=item, order__sent=True
                    )
                ]
            )
            item.items_total = sum(
                [
                    order_item.total
                    for order_item in OrderItem.objects.filter(order__form=self, item=item)
                ]
            )
            item_list.append(item)
        return item_list

    class Meta:
        verbose_name = _("Order form")
        verbose_name_plural = _("Order forms")
        permissions = [("manage_orders_of_form", _("Can manage orders of form"))]


class Order(ExtensibleModel, TimeStampedModel):
    form = models.ForeignKey(
        to=OrderForm, on_delete=models.CASCADE, verbose_name=_("Order form"), related_name="orders"
    )
    full_name = models.CharField(max_length=255, verbose_name=_("First and last name"))
    email = models.EmailField(verbose_name=_("Email"))
    notes = models.TextField(verbose_name=_("Notes"), blank=True)

    submitted = models.BooleanField(default=False, verbose_name=_("Submitted"))
    confirmed = models.BooleanField(default=False, verbose_name=_("Confirmed"))
    confirm_key = models.TextField(verbose_name=_("Confirm key"), blank=True)
    paid = models.BooleanField(default=False, verbose_name=_("Paid"))
    sent = models.BooleanField(default=False, verbose_name=_("Sent"))

    processing_option = models.ForeignKey(
        ProcessingOption,
        models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Selected processing option"),
    )
    shipping_full_name = models.CharField(
        verbose_name=_("First and last name"), max_length=255, blank=True
    )
    second_address_row = models.CharField(
        verbose_name=_("Second address row"), max_length=255, blank=True
    )
    street = models.CharField(verbose_name=_("Street"), max_length=255, blank=True)
    housenumber = models.CharField(verbose_name=_("Housenumber"), max_length=255, blank=True)
    plz = models.CharField(verbose_name=_("PLZ"), max_length=255, blank=True)
    place = models.CharField(verbose_name=_("Place"), max_length=255, blank=True)

    def get_confirm_url(self, request):
        return request.build_absolute_uri(reverse("confirm_order", args=[self.confirm_key]))

    @property
    def email_recipients(self):
        return [f"{self.full_name} <{self.email}>"]

    def send_overview(self, request):
        send_templated_mail(
            template_name="overview",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def confirm(self, request):
        self.confirmed = True
        self.save()
        send_templated_mail(
            template_name="confirmation",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def send_pay_confirmation(self):
        self.paid = True
        self.save()
        send_templated_mail(
            template_name="pay_confirmation",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self},
        )

    def send_confirmation_reminder(self, request):
        send_templated_mail(
            template_name="confirmation_reminder",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self, "confirm_url": self.get_confirm_url(request)},
        )

    def send_pay_reminder(self, request):
        send_templated_mail(
            template_name="pay_reminder",
            from_email=self.form.email_sender,
            recipient_list=self.email_recipients,
            context={"order": self},
        )

    def __str__(self):
        return f"{self.form}: {self.full_name} [{self.created}]"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.confirm_key:
            self.confirm_key = uuid4()

    @property
    def items_count(self):
        return self.items.aggregate(models.Sum("count")).get("count__sum")

    @property
    def processing_price(self):
        return self.processing_option.get_price(self.items_count) if self.processing_option else 0

    @property
    def total(self):
        total = 0
        for item in self.items.all():
            total += item.total
        total += self.processing_price
        return total

    class Meta:
        verbose_name = _("Order")
        verbose_name_plural = _("Orders")
        permissions = [("manage_orders", _("Can manage orders"))]


class OrderItem(ExtensibleModel):
    count = models.IntegerField(verbose_name=_("Count"))
    item = models.ForeignKey(to=Item, on_delete=models.CASCADE, verbose_name=_("Item"))
    order = models.ForeignKey(
        to=Order, on_delete=models.CASCADE, verbose_name=_("Order"), related_name="items"
    )

    @property
    def total(self):
        return self.count * self.item.price



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aleksis',
 'aleksis.apps.order',
 'aleksis.apps.order.migrations',
 'aleksis.apps.order.templatetags',
 'aleksis.apps.order.util']

package_data = \
{'': ['*'],
 'aleksis.apps.order': ['locale/*',
                        'locale/ar/LC_MESSAGES/*',
                        'locale/de_DE/LC_MESSAGES/*',
                        'locale/fr/LC_MESSAGES/*',
                        'locale/la/LC_MESSAGES/*',
                        'locale/nb_NO/LC_MESSAGES/*',
                        'locale/tr_TR/LC_MESSAGES/*',
                        'static/*',
                        'templates/order/*',
                        'templates/templated_email/*']}

install_requires = \
['aleksis-core>=2.0,<3.0',
 'blabel>=0.1.4,<0.2.0',
 'html2text>=2020.1.16,<2021.0.0']

entry_points = \
{'aleksis.app': ['order = aleksis.apps.order.apps:DefaultConfig']}

setup_kwargs = {
    'name': 'aleksis-app-order',
    'version': '0.4',
    'description': 'AlekSIS (School Information System)\u200a—\u200aApp Order (Manage orders)',
    'long_description': 'AlekSIS\u200a—\u200aUnofficial App Order (Manage orders)\n==============================================\n\nAlekSIS\n-------\n\nThis is an **unofficial** application for use with the `AlekSIS`_ platform.\n\nFeatures\n--------\n\nThis application can be used to create order forms and manage orders e. g. for school clothes.\n\nLicence\n-------\n\n::\n\n  Copyright © 2020, 2021 Jonathan Weth <dev@jonathanweth.de>\n  Copyright © 2021 Hangzhi Yu <yuha@katharineum.de>\n\n  Licenced under the EUPL, version 1.2 or later\n\nPlease see the LICENCE.rst file accompanying this distribution for the\nfull licence text or on the `European Union Public Licence`_ website\nhttps://joinup.ec.europa.eu/collection/eupl/guidelines-users-and-developers\n(including all other official language versions).\n\n.. _AlekSIS: https://edugit.org/AlekSIS/AlekSIS\n.. _European Union Public Licence: https://eupl.eu/\n',
    'author': 'Jonathan Weth',
    'author_email': 'dev@jonathanweth.de',
    'maintainer': 'Jonathan Weth',
    'maintainer_email': 'dev@jonathanweth.de',
    'url': 'https://edugit.org/hansegucker/AlekSIS-App-Order',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
